@@ -1,7 +1,7 @@
 /*-------------------------------------------------------------------------
  * hash_helpers.h
  *
- * Copyright (c) 2016, Citus Data, Inc.
+ * Copyright (c) Citus Data, Inc.
  *
  *-------------------------------------------------------------------------
  */
@@ -9,10 +9,14 @@
 #ifndef HASH_HELPERS_H
 #define HASH_HELPERS_H
 
+#include "postgres.h"
+
+#include "distributed/pg_version_constants.h"
+
 #include "utils/hsearch.h"
 
 /* pg12 includes this exact implementation of hash_combine */
-#if PG_VERSION_NUM < 120000
+#if PG_VERSION_NUM < PG_VERSION_12
 
 /*
  * Combine two hash values, resulting in another hash value, with decent bit
@@ -33,4 +37,15 @@ hash_combine(uint32 a, uint32 b)
 
 extern void hash_delete_all(HTAB *htab);
 
+/*
+ * foreach_htab -
+ *	  a convenience macro which loops through a HTAB
+ */
+
+#define foreach_htab(var, status, htab) \
+	hash_seq_init((status), (htab)); \
+	for ((var) = hash_seq_search(status); \
+		 (var) != NULL; \
+		 (var) = hash_seq_search(status))
+
 #endif
