@@ -4,7 +4,7 @@
  *	  Type and function declarations used to sync metadata across all
  *	  workers.
  *
- * Copyright (c) 2016, Citus Data, Inc.
+ * Copyright (c) Citus Data, Inc.
  *
  *-------------------------------------------------------------------------
  */
@@ -28,32 +28,36 @@ typedef enum
 } MetadataSyncResult;
 
 /* Functions declarations for metadata syncing */
-extern void StartMetadatSyncToNode(char *nodeNameString, int32 nodePort);
+extern void StartMetadataSyncToNode(const char *nodeNameString, int32 nodePort);
 extern bool ClusterHasKnownMetadataWorkers(void);
 extern bool ShouldSyncTableMetadata(Oid relationId);
 extern List * MetadataCreateCommands(void);
-extern List * GetDistributedTableDDLEvents(Oid relationId);
 extern List * MetadataDropCommands(void);
-extern char * DistributionCreateCommand(DistTableCacheEntry *cacheEntry);
-extern char * DistributionDeleteCommand(char *schemaName,
-										char *tableName);
+extern char * DistributionCreateCommand(CitusTableCacheEntry *cacheEntry);
+extern char * DistributionDeleteCommand(const char *schemaName,
+										const char *tableName);
 extern char * TableOwnerResetCommand(Oid distributedRelationId);
 extern char * NodeListInsertCommand(List *workerNodeList);
 extern List * ShardListInsertCommand(List *shardIntervalList);
-extern List * ShardDeleteCommandList(ShardInterval *shardInterval);
 extern char * NodeDeleteCommand(uint32 nodeId);
 extern char * NodeStateUpdateCommand(uint32 nodeId, bool isActive);
+extern char * ShouldHaveShardsUpdateCommand(uint32 nodeId, bool shouldHaveShards);
 extern char * ColocationIdUpdateCommand(Oid relationId, uint32 colocationId);
 extern char * CreateSchemaDDLCommand(Oid schemaId);
+extern List * GrantOnSchemaDDLCommands(Oid schemaId);
 extern char * PlacementUpsertCommand(uint64 shardId, uint64 placementId, int shardState,
 									 uint64 shardLength, int32 groupId);
 extern void CreateTableMetadataOnWorkers(Oid relationId);
-extern void MarkNodeHasMetadata(char *nodeName, int32 nodePort, bool hasMetadata);
-extern void MarkNodeMetadataSynced(char *nodeName, int32 nodePort, bool synced);
-extern MetadataSyncResult SyncMetadataToNodes(void);
-extern bool SendOptionalCommandListToWorkerInTransaction(char *nodeName, int32 nodePort,
-														 char *nodeUser,
+extern void MarkNodeHasMetadata(const char *nodeName, int32 nodePort, bool hasMetadata);
+extern void MarkNodeMetadataSynced(const char *nodeName, int32 nodePort, bool synced);
+extern BackgroundWorkerHandle * SpawnSyncMetadataToNodes(Oid database, Oid owner);
+extern bool SendOptionalCommandListToWorkerInTransaction(const char *nodeName, int32
+														 nodePort,
+														 const char *nodeUser,
 														 List *commandList);
+extern void SyncMetadataToNodesMain(Datum main_arg);
+extern void SignalMetadataSyncDaemon(Oid database, int sig);
+extern bool ShouldInitiateMetadataSync(bool *lockFailure);
 
 #define DELETE_ALL_NODES "TRUNCATE pg_dist_node CASCADE"
 #define REMOVE_ALL_CLUSTERED_TABLES_COMMAND \
