@@ -4,7 +4,7 @@
  *	  Type and function declarations used in creating the distributed execution
  *	  plan.
  *
- * Copyright (c) 2012-2016, Citus Data, Inc.
+ * Copyright (c) Citus Data, Inc.
  *
  * $Id$
  *
@@ -15,12 +15,17 @@
 #define MULTI_PHYSICAL_PLANNER_H
 
 #include "postgres.h"
+
+#include "distributed/pg_version_constants.h"
+
 #include "c.h"
 
 #include "datatype/timestamp.h"
 #include "distributed/citus_nodes.h"
 #include "distributed/errormessage.h"
-#include "distributed/master_metadata_utility.h"
+#include "distributed/log_utils.h"
+#include "distributed/metadata_utility.h"
+#include "distributed/worker_manager.h"
 #include "distributed/multi_logical_planner.h"
 #include "distributed/distributed_planner.h"
 #include "lib/stringinfo.h"
@@ -29,7 +34,6 @@
 
 
 /* Definitions local to the physical planner */
-#define ARRAY_OUT_FUNC_ID 751
 #define NON_PRUNABLE_JOIN -1
 #define RESERVED_HASHED_COLUMN_ID MaxAttrNumber
 #define MERGE_COLUMN_FORMAT "merge_column_%u"
@@ -41,9 +45,8 @@
  (" UINT64_FORMAT ", %d, %s, '%s', '%s'::regtype, %s)"
 #define MERGE_FILES_INTO_TABLE_COMMAND "SELECT worker_merge_files_into_table \
  (" UINT64_FORMAT ", %d, '%s', '%s')"
-#define MERGE_FILES_AND_RUN_QUERY_COMMAND \
-	"SELECT worker_merge_files_and_run_query(" UINT64_FORMAT ", %d, %s, %s)"
 
+extern int RepartitionJoinBucketCountPerNode;
 
 typedef enum CitusRTEKind
 {
@@ -55,7 +58,7 @@ typedef enum CitusRTEKind
 	CITUS_RTE_VALUES = RTE_VALUES,      /* VALUES (<exprlist>), (<exprlist>), ... */
 	CITUS_RTE_CTE = RTE_CTE,            /* common table expr (WITH list element) */
 	CITUS_RTE_NAMEDTUPLESTORE = RTE_NAMEDTUPLESTORE, /* tuplestore, e.g. for triggers */
-#if (PG_VERSION_NUM >= 120000)
+#if (PG_VERSION_NUM >= PG_VERSION_12)
 	CITUS_RTE_RESULT = RTE_RESULT,      /* RTE represents an empty FROM clause */
 #endif
 	CITUS_RTE_SHARD,
@@ -76,16 +79,15 @@ typedef enum
 /* Enumeration that defines different task types */
 typedef enum
 {
-	TASK_TYPE_INVALID_FIRST = 0,
-	SQL_TASK = 1,
-	MAP_TASK = 2,
-	MERGE_TASK = 3,
-	MAP_OUTPUT_FETCH_TASK = 4,
-	MERGE_FETCH_TASK = 5,
-	MODIFY_TASK = 6,
-	ROUTER_TASK = 7,
-	DDL_TASK = 8,
-	VACUUM_ANALYZE_TASK = 9
+	TASK_TYPE_INVALID_FIRST,
+	READ_TASK,
+	MAP_TASK,
+	MERGE_TASK,
+	MAP_OUTPUT_FETCH_TASK,
+	MERGE_FETCH_TASK,
+	MODIFY_TASK,
+	DDL_TASK,
+	VACUUM_ANALYZE_TASK
 } TaskType;
 
 
@@ -118,6 +120,21 @@ typedef enum RowModifyLevel
 	ROW_MODIFY_NONCOMMUTATIVE = 3
 } RowModifyLevel;
 
+
+/*
+ * LocalPlannedStatement represents a local plan of a shard. The scope
+ * for the LocalPlannedStatement is Task.
+ */
+typedef struct LocalPlannedStatement
+{
+	CitusNode type;
+
+	uint64 shardId;
+	uint32 localGroupId;
+	PlannedStmt *localPlan;
+} LocalPlannedStatement;
+
+
 /*
  * Job represents a logical unit of work that contains one set of data transfers
  * in our physical plan. The physical planner maps each SQL query into one or
@@ -131,11 +148,21 @@ typedef struct Job
 	uint64 jobId;
 	Query *jobQuery;
 	List *taskList;
-	List *dependedJobList;
+	List *dependentJobList;
 	bool subqueryPushdown;
-	bool requiresMasterEvaluation; /* only applies to modify jobs */
+	bool requiresCoordinatorEvaluation; /* only applies to modify jobs */
 	bool deferredPruning;
 	Const *partitionKeyValue;
+
+	/* for local shard queries, we may save the local plan here */
+	List *localPlannedStatements;
+
+	/*
+	 * When we evaluate functions and parameters in jobQuery then we
+	 * should no longer send the list of parameters along with the
+	 * query.
+	 */
+	bool parametersInJobQueryResolved;
 } Job;
 
 
@@ -153,27 +180,60 @@ typedef struct MapMergeJob
 	List *mergeTaskList;
 } MapMergeJob;
 
+typedef enum TaskQueryType
+{
+	TASK_QUERY_NULL,
+	TASK_QUERY_TEXT,
+	TASK_QUERY_OBJECT,
+	TASK_QUERY_TEXT_LIST
+} TaskQueryType;
 
-/*
- * Task represents an executable unit of work. We conceptualize our tasks into
- * compute and data fetch task types. SQL, map, and merge tasks are considered
- * as compute tasks; and map fetch, and merge fetch tasks are data
- * fetch tasks. We also forward declare the task execution struct here to avoid
- * including the executor header files.
- *
- * We currently do not take replication model into account for tasks other
- * than modifications. When it is set to REPLICATION_MODEL_2PC, the execution
- * of the modification task is done with two-phase commit. Set it to
- * REPLICATION_MODEL_INVALID if it is not relevant for the task.
- *
- * NB: Changing this requires also changing _outTask in citus_outfuncs and _readTask
- * in citus_readfuncs to correctly (de)serialize this struct.
- *
- * INSERT ... SELECT queries and modify queries with subqueries or multiple tables
- * set modifyWithSubquery to true. We need to use it to take the necessary locks
- * to get consistent results for subqueries.
- */
-typedef struct TaskExecution TaskExecution;
+typedef struct TaskQuery
+{
+	TaskQueryType queryType;
+
+	union
+	{
+		/*
+		 * For most queries jobQueryReferenceForLazyDeparsing and/or queryStringLazy is not
+		 * NULL. This means we have a single query for all placements.
+		 *
+		 * If this is not the case, the length of perPlacementQueryStrings is
+		 * non-zero and equal to length of taskPlacementList. Like this it can
+		 * assign a different query for each placement. We need this flexibility
+		 * when a query should return node specific values. For example, on which
+		 * node did we succeed storing some result files?
+		 *
+		 * jobQueryReferenceForLazyDeparsing is only not null when the planner thinks the
+		 * query could possibly be locally executed. In that case deparsing+parsing
+		 * the query might not be necessary, so we do that lazily.
+		 *
+		 * jobQueryReferenceForLazyDeparsing should only be set by using SetTaskQueryIfShouldLazyDeparse()
+		 */
+		Query *jobQueryReferenceForLazyDeparsing;
+
+		/*
+		 * In almost all cases queryStringLazy should be read only indirectly by
+		 * using TaskQueryString(). This will populate the field if only the
+		 * jobQueryReferenceForLazyDeparsing field is not NULL.
+		 *
+		 * This field should only be set by using SetTaskQueryString() (or as a
+		 * side effect from TaskQueryString()). Otherwise it might not be in sync
+		 * with jobQueryReferenceForLazyDeparsing.
+		 */
+		char *queryStringLazy;
+
+		/*
+		 * queryStringList contains query strings. They should be
+		 * run sequentially. The concatenated version of this list
+		 * will already be set for queryStringLazy, this can be useful
+		 * when we want to access each query string.
+		 */
+		List *queryStringList;
+	}data;
+}TaskQuery;
+
+struct TupleDestination;
 
 typedef struct Task
 {
@@ -181,16 +241,28 @@ typedef struct Task
 	TaskType taskType;
 	uint64 jobId;
 	uint32 taskId;
-	char *queryString;
+
+	/*
+	 * taskQuery contains query string information. The way we get queryString can be different
+	 * so this is abstracted with taskQuery.
+	 */
+	TaskQuery taskQuery;
+
+	/*
+	 * A task can have multiple queries, in which case queryCount will be > 1. If
+	 * a task has more one query, then taskQuery->queryType == TASK_QUERY_TEXT_LIST.
+	 */
+	int queryCount;
+
+	Oid anchorDistributedTableId;     /* only applies to insert tasks */
 	uint64 anchorShardId;       /* only applies to compute tasks */
 	List *taskPlacementList;    /* only applies to compute tasks */
-	List *dependedTaskList;     /* only applies to compute tasks */
+	List *dependentTaskList;     /* only applies to compute tasks */
 
 	uint32 partitionId;
 	uint32 upstreamTaskId;         /* only applies to data fetch tasks */
 	ShardInterval *shardInterval;  /* only applies to merge tasks */
 	bool assignmentConstrained;    /* only applies to merge tasks */
-	TaskExecution *taskExecution;  /* used by task tracker executor */
 	char replicationModel;         /* only applies to modify tasks */
 
 	/*
@@ -213,6 +285,49 @@ typedef struct Task
 	List *relationShardList;
 
 	List *rowValuesLists;          /* rows to use when building multi-row INSERT */
+
+	/*
+	 * Used only when local execution happens. Indicates that this task is part of
+	 * both local and remote executions. We use "or" in the field name because this
+	 * is set to true for both the remote and local tasks generated for such
+	 * executions. The most common example is modifications to reference tables where
+	 * the task splitted into local and remote tasks.
+	 */
+	bool partiallyLocalOrRemote;
+
+	/*
+	 * When we evaluate functions and parameters in the query string then
+	 * we should no longer send the list of parameters long with the
+	 * query.
+	 */
+	bool parametersInQueryStringResolved;
+
+	/*
+	 * Destination of tuples generated as a result of executing this task. Can be
+	 * NULL, in which case executor might use a default destination.
+	 */
+	struct TupleDestination *tupleDest;
+
+	/*
+	 * totalReceivedTupleData only counts the data for a single placement. So
+	 * for RETURNING DML this is not really correct. This is used by
+	 * EXPLAIN ANALYZE, to display the amount of received bytes. The local execution
+	 * does not increment this value, so only used for remote execution.
+	 */
+	uint64 totalReceivedTupleData;
+
+	/*
+	 * EXPLAIN ANALYZE output fetched from worker. This is saved to be used later
+	 * by RemoteExplain().
+	 */
+	char *fetchedExplainAnalyzePlan;
+	int fetchedExplainAnalyzePlacementIndex;
+
+	/*
+	 * Execution Duration fetched from worker. This is saved to be used later by
+	 * ExplainTaskList().
+	 */
+	double fetchedExplainAnalyzeExecutionDuration;
 } Task;
 
 
@@ -240,6 +355,22 @@ typedef struct JoinSequenceNode
 } JoinSequenceNode;
 
 
+/*
+ * InsertSelectMethod represents the method to use for INSERT INTO ... SELECT
+ * queries.
+ *
+ * Note that there is a third method which is not represented here, which is
+ * pushing down the INSERT INTO ... SELECT to workers. This method is executed
+ * similar to other distributed queries and doesn't need a special execution
+ * code, so we don't need to represent it here.
+ */
+typedef enum InsertSelectMethod
+{
+	INSERT_SELECT_VIA_COORDINATOR,
+	INSERT_SELECT_REPARTITION
+} InsertSelectMethod;
+
+
 /*
  * DistributedPlan contains all information necessary to execute a
  * distribute query.
@@ -254,17 +385,17 @@ typedef struct DistributedPlan
 	/* specifies nature of modifications in query */
 	RowModifyLevel modLevel;
 
-	/* specifies whether a DML command has a RETURNING */
-	bool hasReturning;
-
-	/* a router executable query is executed entirely on a worker */
-	bool routerExecutable;
+	/*
+	 * specifies whether plan returns results,
+	 * either as a SELECT or a DML which has RETURNING.
+	 */
+	bool expectResults;
 
 	/* job tree containing the tasks to be executed on workers */
 	Job *workerJob;
 
 	/* local query that merges results from the workers */
-	Query *masterQuery;
+	Query *combineQuery;
 
 	/* query identifier (copied from the top-level PlannedStmt) */
 	uint64 queryId;
@@ -272,15 +403,15 @@ typedef struct DistributedPlan
 	/* which relations are accessed by this distributed plan */
 	List *relationIdList;
 
-	/* SELECT query in an INSERT ... SELECT via the coordinator */
-	Query *insertSelectSubquery;
-
-	/* target list of an INSERT ... SELECT via the coordinator */
-	List *insertTargetList;
-
 	/* target relation of a modification */
 	Oid targetRelationId;
 
+	/*
+	 * INSERT .. SELECT via the coordinator or repartition */
+	Query *insertSelectQuery;
+	PlannedStmt *selectPlanForInsertSelect;
+	InsertSelectMethod insertSelectMethod;
+
 	/*
 	 * If intermediateResultIdPrefix is non-null, an INSERT ... SELECT
 	 * via the coordinator is written to a set of intermediate results
@@ -294,6 +425,26 @@ typedef struct DistributedPlan
 	/* list of subplans to execute before the distributed query */
 	List *subPlanList;
 
+	/*
+	 * List of subPlans that are used in the DistributedPlan
+	 * Note that this is different that "subPlanList" field which
+	 * contains the subplans generated as part of the DistributedPlan.
+	 *
+	 * On the other hand, usedSubPlanNodeList keeps track of which subPlans
+	 * are used within this distributed plan as a list of
+	 * UsedDistributedSubPlan pointers.
+	 *
+	 * The list may contain duplicates if the subplan is referenced multiple
+	 * times (e.g. a CTE appears in the query tree multiple times).
+	 */
+	List *usedSubPlanNodeList;
+
+	/*
+	 * When the query is very simple such that we don't need to call
+	 * standard_planner(). See FastPathRouterQuery() for the definition.
+	 */
+	bool fastPathRouterPlan;
+
 	/*
 	 * NULL if this a valid plan, an error description otherwise. This will
 	 * e.g. be set if SQL features are present that a planner doesn't support,
@@ -315,9 +466,41 @@ typedef struct DistributedSubPlan
 
 	uint32 subPlanId;
 	PlannedStmt *plan;
+
+	/* EXPLAIN ANALYZE instrumentations */
+	uint64 bytesSentPerWorker;
+	uint32 remoteWorkerCount;
+	double durationMillisecs;
+	bool writeLocalFile;
 } DistributedSubPlan;
 
 
+/* defines how a subplan is used by a distributed query */
+typedef enum SubPlanAccessType
+{
+	SUBPLAN_ACCESS_NONE,
+	SUBPLAN_ACCESS_LOCAL,
+	SUBPLAN_ACCESS_REMOTE,
+	SUBPLAN_ACCESS_ANYWHERE
+} SubPlanAccessType;
+
+
+/*
+ * UsedDistributedSubPlan contains information about a subPlan that is used in a
+ * distributed plan.
+ */
+typedef struct UsedDistributedSubPlan
+{
+	CitusNode type;
+
+	/* subplan used by the distributed query */
+	char *subPlanId;
+
+	/* how the subplan is used by a distributed query */
+	SubPlanAccessType accessType;
+} UsedDistributedSubPlan;
+
+
 /* OperatorCacheEntry contains information for each element in OperatorCache */
 typedef struct OperatorCacheEntry
 {
@@ -331,6 +514,10 @@ typedef struct OperatorCacheEntry
 } OperatorCacheEntry;
 
 
+/* Named function pointer type for reordering Task lists */
+typedef List *(*ReorderFunction)(List *);
+
+
 /* Config variable managed via guc.c */
 extern int TaskAssignmentPolicy;
 extern bool EnableUniqueJobIds;
@@ -344,6 +531,10 @@ extern Task * CreateBasicTask(uint64 jobId, uint32 taskId, TaskType taskType,
 							  char *queryString);
 
 extern OpExpr * MakeOpExpression(Var *variable, int16 strategyNumber);
+extern Node *  WrapUngroupedVarsInAnyValueAggregate(Node *expression,
+													List *groupClauseList,
+													List *targetList,
+													bool checkExpressionEquality);
 
 /*
  * Function declarations for building, updating constraints and simple operator
@@ -351,41 +542,52 @@ extern OpExpr * MakeOpExpression(Var *variable, int16 strategyNumber);
  */
 extern Node * BuildBaseConstraint(Var *column);
 extern void UpdateConstraint(Node *baseConstraint, ShardInterval *shardInterval);
+extern bool BinaryOpExpression(Expr *clause, Node **leftOperand, Node **rightOperand);
 extern bool SimpleOpExpression(Expr *clause);
-extern bool OpExpressionContainsColumn(OpExpr *operatorExpression, Var *partitionColumn);
 
 /* helper functions */
 extern Var * MakeInt4Column(void);
-extern Const * MakeInt4Constant(Datum constantValue);
 extern int CompareShardPlacements(const void *leftElement, const void *rightElement);
 extern bool ShardIntervalsOverlap(ShardInterval *firstInterval,
 								  ShardInterval *secondInterval);
 extern bool CoPartitionedTables(Oid firstRelationId, Oid secondRelationId);
 extern ShardInterval ** GenerateSyntheticShardIntervalArray(int partitionCount);
 extern RowModifyLevel RowModifyLevelForQuery(Query *query);
+extern StringInfo ArrayObjectToString(ArrayType *arrayObject,
+									  Oid columnType, int32 columnTypeMod);
 
 
 /* function declarations for Task and Task list operations */
 extern bool TasksEqual(const Task *a, const Task *b);
-extern List * TaskListAppendUnique(List *list, Task *task);
-extern List * TaskListConcatUnique(List *list1, List *list2);
 extern bool TaskListMember(const List *taskList, const Task *task);
 extern List * TaskListDifference(const List *list1, const List *list2);
 extern List * AssignAnchorShardTaskList(List *taskList);
 extern List * FirstReplicaAssignTaskList(List *taskList);
 extern List * RoundRobinAssignTaskList(List *taskList);
-extern List * RoundRobinReorder(Task *task, List *placementList);
+extern List * RoundRobinReorder(List *placementList);
+extern void SetPlacementNodeMetadata(ShardPlacement *placement, WorkerNode *workerNode);
 extern int CompareTasksByTaskId(const void *leftElement, const void *rightElement);
+extern int CompareTasksByExecutionDuration(const void *leftElement, const
+										   void *rightElement);
 
 /* function declaration for creating Task */
 extern List * QueryPushdownSqlTaskList(Query *query, uint64 jobId,
 									   RelationRestrictionContext *
 									   relationRestrictionContext,
 									   List *prunedRelationShardList, TaskType taskType,
-									   bool modifyRequiresMasterEvaluation);
+									   bool modifyRequiresCoordinatorEvaluation);
 
 /* function declarations for managing jobs */
 extern uint64 UniqueJobId(void);
 
 
+extern List * DerivedColumnNameList(uint32 columnCount, uint64 generatingJobId);
+extern RangeTblEntry * DerivedRangeTableEntry(MultiNode *multiNode, List *columnList,
+											  List *tableIdList,
+											  List *funcColumnNames,
+											  List *funcColumnTypes,
+											  List *funcColumnTypeMods,
+											  List *funcCollations);
+
+
 #endif   /* MULTI_PHYSICAL_PLANNER_H */
