@@ -3,13 +3,13 @@
  *
  * Function declartions for transaction access tracking.
  *
- * Copyright (c) 2018, Citus Data, Inc.
+ * Copyright (c) Citus Data, Inc.
  */
 
 #ifndef RELATION_ACCESS_TRACKING_H_
 #define RELATION_ACCESS_TRACKING_H_
 
-#include "distributed/master_metadata_utility.h"
+#include "distributed/metadata_utility.h"
 #include "distributed/multi_physical_planner.h" /* access Task struct */
 #include "distributed/placement_connection.h"
 
@@ -36,8 +36,8 @@ typedef enum RelationAccessMode
 
 extern void AllocateRelationAccessHash(void);
 extern void ResetRelationAccessHash(void);
-extern void RecordRelationAccessIfReferenceTable(Oid relationId,
-												 ShardPlacementAccessType accessType);
+extern void RecordRelationAccessIfNonDistTable(Oid relationId,
+											   ShardPlacementAccessType accessType);
 extern void RecordParallelRelationAccessForTaskList(List *taskList);
 extern void RecordParallelSelectAccess(Oid relationId);
 extern void RecordParallelModifyAccess(Oid relationId);
