@@ -3,7 +3,7 @@
  * remote_commands.h
  *	  Helpers to execute commands on remote nodes, over libpq.
  *
- * Copyright (c) 2016, Citus Data, Inc.
+ * Copyright (c) Citus Data, Inc.
  *
  *-------------------------------------------------------------------------
  */
@@ -28,7 +28,6 @@ extern void ForgetResults(MultiConnection *connection);
 extern bool ClearResults(MultiConnection *connection, bool raiseErrors);
 extern bool ClearResultsDiscardWarnings(MultiConnection *connection, bool raiseErrors);
 extern bool ClearResultsIfReady(MultiConnection *connection);
-extern bool SqlStateMatchesCategory(char *sqlStateString, int category);
 
 /* report errors & warnings */
 extern void ReportConnectionError(MultiConnection *connection, int elevel);
@@ -48,7 +47,8 @@ extern int ExecuteOptionalRemoteCommand(MultiConnection *connection,
 extern int SendRemoteCommand(MultiConnection *connection, const char *command);
 extern int SendRemoteCommandParams(MultiConnection *connection, const char *command,
 								   int parameterCount, const Oid *parameterTypes,
-								   const char *const *parameterValues);
+								   const char *const *parameterValues,
+								   bool binaryResults);
 extern List * ReadFirstColumnAsText(PGresult *queryResult);
 extern PGresult * GetRemoteCommandResult(MultiConnection *connection,
 										 bool raiseInterrupts);
