@@ -3,7 +3,7 @@
  * resource_lock.h
  *	  Locking Infrastructure for Citus.
  *
- * Copyright (c) 2012-2016, Citus Data, Inc.
+ * Copyright (c) Citus Data, Inc.
  *-------------------------------------------------------------------------
  */
 
@@ -36,9 +36,16 @@ typedef enum AdvisoryLocktagClass
 	ADV_LOCKTAG_CLASS_CITUS_SHARD_METADATA = 4,
 	ADV_LOCKTAG_CLASS_CITUS_SHARD = 5,
 	ADV_LOCKTAG_CLASS_CITUS_JOB = 6,
-	ADV_LOCKTAG_CLASS_CITUS_REBALANCE_COLOCATION = 7
+	ADV_LOCKTAG_CLASS_CITUS_REBALANCE_COLOCATION = 7,
+	ADV_LOCKTAG_CLASS_CITUS_COLOCATED_SHARDS_METADATA = 8,
+	ADV_LOCKTAG_CLASS_CITUS_OPERATIONS = 9
 } AdvisoryLocktagClass;
 
+/* CitusOperations has constants for citus operations */
+typedef enum CitusOperations
+{
+	CITUS_TRANSACTION_RECOVERY = 0
+} CitusOperations;
 
 /* reuse advisory lock, but with different, unused field 4 (4)*/
 #define SET_LOCKTAG_SHARD_METADATA_RESOURCE(tag, db, shardid) \
@@ -48,6 +55,14 @@ typedef enum AdvisoryLocktagClass
 						 (uint32) (shardid), \
 						 ADV_LOCKTAG_CLASS_CITUS_SHARD_METADATA)
 
+#define SET_LOCKTAG_COLOCATED_SHARDS_METADATA_RESOURCE(tag, db, colocationId, \
+													   shardIntervalIndex) \
+	SET_LOCKTAG_ADVISORY(tag, \
+						 db, \
+						 (uint32) shardIntervalIndex, \
+						 (uint32) colocationId, \
+						 ADV_LOCKTAG_CLASS_CITUS_COLOCATED_SHARDS_METADATA)
+
 /* reuse advisory lock, but with different, unused field 4 (5)*/
 #define SET_LOCKTAG_SHARD_RESOURCE(tag, db, shardid) \
 	SET_LOCKTAG_ADVISORY(tag, \
@@ -65,7 +80,7 @@ typedef enum AdvisoryLocktagClass
 						 ADV_LOCKTAG_CLASS_CITUS_JOB)
 
 /* reuse advisory lock, but with different, unused field 4 (7)
- * Also it has the the database hardcoded to MyDatabaseId, to ensure the locks
+ * Also it has the database hardcoded to MyDatabaseId, to ensure the locks
  * are local to each database */
 #define SET_LOCKTAG_REBALANCE_COLOCATION(tag, colocationOrTableId) \
 	SET_LOCKTAG_ADVISORY(tag, \
@@ -74,10 +89,17 @@ typedef enum AdvisoryLocktagClass
 						 (uint32) (colocationOrTableId), \
 						 ADV_LOCKTAG_CLASS_CITUS_REBALANCE_COLOCATION)
 
+/* advisory lock for citus operations, also it has the database hardcoded to MyDatabaseId,
+ * to ensure the locks are local to each database */
+#define SET_LOCKTAG_CITUS_OPERATION(tag, operationId) \
+	SET_LOCKTAG_ADVISORY(tag, \
+						 MyDatabaseId, \
+						 (uint32) 0, \
+						 (uint32) operationId, \
+						 ADV_LOCKTAG_CLASS_CITUS_OPERATIONS)
 
 /* Lock shard/relation metadata for safe modifications */
 extern void LockShardDistributionMetadata(int64 shardId, LOCKMODE lockMode);
-extern bool TryLockShardDistributionMetadata(int64 shardId, LOCKMODE lockMode);
 extern void LockShardListMetadataOnWorkers(LOCKMODE lockmode, List *shardIntervalList);
 extern void BlockWritesToShardList(List *shardList);
 
@@ -87,24 +109,26 @@ extern void LockReferencedReferenceShardDistributionMetadata(uint64 shardId,
 
 /* Lock shard data, for DML commands or remote fetches */
 extern void LockShardResource(uint64 shardId, LOCKMODE lockmode);
-extern void UnlockShardResource(uint64 shardId, LOCKMODE lockmode);
 
 /* Lock a job schema or partition task directory */
 extern void LockJobResource(uint64 jobId, LOCKMODE lockmode);
 extern void UnlockJobResource(uint64 jobId, LOCKMODE lockmode);
 
+/* Lock a co-location group */
+extern void LockColocationId(int colocationId, LOCKMODE lockMode);
+extern void UnlockColocationId(int colocationId, LOCKMODE lockMode);
+
 /* Lock multiple shards for safe modification */
 extern void LockShardListMetadata(List *shardIntervalList, LOCKMODE lockMode);
 extern void LockShardsInPlacementListMetadata(List *shardPlacementList,
 											  LOCKMODE lockMode);
+
+extern void LockTransactionRecovery(LOCKMODE lockMode);
+
 extern void SerializeNonCommutativeWrites(List *shardIntervalList, LOCKMODE lockMode);
 extern void LockRelationShardResources(List *relationShardList, LOCKMODE lockMode);
 extern List * GetSortedReferenceShardIntervals(List *relationList);
 
-/* Lock partitions of partitioned table */
-extern void LockPartitionsInRelationList(List *relationIdList, LOCKMODE lockmode);
-extern void LockPartitionRelations(Oid relationId, LOCKMODE lockMode);
-
 /* Lock parent table's colocated shard resource */
 extern void LockParentShardResourceIfPartition(uint64 shardId, LOCKMODE lockMode);
 
