@@ -3,7 +3,7 @@
  * shard_pruning.h
  *   Shard pruning infrastructure.
  *
- * Copyright (c) 2014-2017, Citus Data, Inc.
+ * Copyright (c) Citus Data, Inc.
  *
  *-------------------------------------------------------------------------
  */
@@ -20,5 +20,7 @@
 extern List * PruneShards(Oid relationId, Index rangeTableId, List *whereClauseList,
 						  Const **partitionValueConst);
 extern bool ContainsFalseClause(List *whereClauseList);
-
+extern Const * TransformPartitionRestrictionValue(Var *partitionColumn,
+												  Const *restrictionValue,
+												  bool missingOk);
 #endif /* SHARD_PRUNING_H_ */
